@@ -30,8 +30,6 @@
 #include <gtk/gtk.h>
 #include <gdk/gdkkeysyms.h>
 
-#include <dbus/dbus-glib.h>
-
 #include <bluetooth-client.h>
 #include <bluetooth-client-private.h>
 #include <bluetooth-chooser.h>
@@ -155,8 +153,8 @@ update_random_pincode (void)
 }
 
 static gboolean
-pincode_callback (DBusGMethodInvocation *context,
-		  DBusGProxy *device,
+pincode_callback (GDBusMethodInvocation *invocation,
+		  GDBusProxy *device,
 		  gpointer user_data)
 {
 	target_ssp = FALSE;
@@ -164,7 +162,7 @@ pincode_callback (DBusGMethodInvocation *context,
 	/* Only show the pincode page if the pincode isn't automatic */
 	if (automatic_pincode == FALSE)
 		gtk_assistant_set_current_page (window_assistant, PAGE_SETUP);
-	dbus_g_method_return(context, pincode);
+	g_dbus_method_invocation_return_value (invocation, g_variant_new ("(s)", pincode));
 
 	return TRUE;
 }
@@ -194,7 +192,7 @@ void
 does_not_match_cb (GtkButton *button,
 		   gpointer user_data)
 {
-	DBusGMethodInvocation *context;
+	GDBusMethodInvocation *invocation;
 	GError *error = NULL;
 	char *text;
 
@@ -208,33 +206,34 @@ does_not_match_cb (GtkButton *button,
 	gtk_label_set_text(GTK_LABEL(label_failure), text);
 	g_free(text);
 
-	context = g_object_get_data (G_OBJECT (button), "context");
-	g_error_new(AGENT_ERROR, AGENT_ERROR_REJECT,
-		    "Agent callback cancelled");
-	dbus_g_method_return(context, error);
+	invocation = g_object_get_data (G_OBJECT (button), "invocation");
+	error = g_error_new (AGENT_ERROR, AGENT_ERROR_REJECT,
+			     "Agent callback cancelled");
+	g_dbus_method_invocation_return_gerror (invocation, error);
+	g_error_free (error);
 
-	g_object_set_data (G_OBJECT(does_not_match_button), "context", NULL);
-	g_object_set_data (G_OBJECT(matches_button), "context", NULL);
+	g_object_set_data (G_OBJECT(does_not_match_button), "invocation", NULL);
+	g_object_set_data (G_OBJECT(matches_button), "invocation", NULL);
 }
 
 void
 matches_cb (GtkButton *button,
 	    gpointer user_data)
 {
-	DBusGMethodInvocation *context;
+	GDBusMethodInvocation *invocation;
 
-	context = g_object_get_data (G_OBJECT (button), "context");
+	invocation = g_object_get_data (G_OBJECT (button), "invocation");
 	gtk_widget_set_sensitive (does_not_match_button, FALSE);
 	gtk_widget_set_sensitive (matches_button, FALSE);
-	dbus_g_method_return(context, "");
+	g_dbus_method_invocation_return_value (invocation, g_variant_new ("(s)", ""));
 
-	g_object_set_data (G_OBJECT(does_not_match_button), "context", NULL);
-	g_object_set_data (G_OBJECT(matches_button), "context", NULL);
+	g_object_set_data (G_OBJECT(does_not_match_button), "invocation", NULL);
+	g_object_set_data (G_OBJECT(matches_button), "invocation", NULL);
 }
 
 static gboolean
-confirm_callback (DBusGMethodInvocation *context,
-		  DBusGProxy *device,
+confirm_callback (GDBusMethodInvocation *invocation,
+		  GDBusProxy *device,
 		  guint pin,
 		  gpointer user_data)
 {
@@ -250,19 +249,19 @@ confirm_callback (DBusGMethodInvocation *context,
 	g_free (label);
 
 	gtk_widget_show (label_ssp_pin);
-	str = g_strdup_printf ("%d", pin);
+	str = g_strdup_printf ("%06d", pin);
 	set_large_label (GTK_LABEL (label_ssp_pin), str);
 	g_free (str);
 
-	g_object_set_data (G_OBJECT(does_not_match_button), "context", context);
-	g_object_set_data (G_OBJECT(matches_button), "context", context);
+	g_object_set_data (G_OBJECT(does_not_match_button), "invocation", invocation);
+	g_object_set_data (G_OBJECT(matches_button), "invocation", invocation);
 
 	return TRUE;
 }
 
 static gboolean
-display_callback (DBusGMethodInvocation *context,
-		  DBusGProxy *device,
+display_callback (GDBusMethodInvocation *invocation,
+		  GDBusProxy *device,
 		  guint pin,
 		  guint entered,
 		  gpointer user_data)
@@ -273,7 +272,7 @@ display_callback (DBusGMethodInvocation *context,
 	target_ssp = TRUE;
 	gtk_assistant_set_current_page (window_assistant, PAGE_SSP_SETUP);
 
-	code = g_strdup_printf("%d", pin);
+	code = g_strdup_printf("%06d", pin);
 
 	if (entered > 0) {
 		GtkEntry *entry;
@@ -306,13 +305,13 @@ display_callback (DBusGMethodInvocation *context,
 	g_free(done);
 	g_free(code);
 
-	dbus_g_method_return(context);
+	g_dbus_method_invocation_return_value (invocation, NULL);
 
 	return TRUE;
 }
 
 static gboolean
-cancel_callback (DBusGMethodInvocation *context,
+cancel_callback (GDBusMethodInvocation *invocation,
 		 gpointer user_data)
 {
 	gchar *text;
@@ -329,7 +328,7 @@ cancel_callback (DBusGMethodInvocation *context,
 	gtk_label_set_text(GTK_LABEL(label_failure), text);
 	g_free(text);
 
-	dbus_g_method_return(context);
+	g_dbus_method_invocation_return_value (invocation, NULL);
 
 	return TRUE;
 }
@@ -340,19 +339,21 @@ typedef struct {
 } ConnectData;
 
 static void
-connect_callback (BluetoothClient *_client,
-		  gboolean success,
-		  gpointer user_data)
+connect_callback (GObject      *source_object,
+		  GAsyncResult *res,
+		  gpointer      user_data)
 {
 	ConnectData *data = (ConnectData *) user_data;
+	gboolean success;
+
+	success = bluetooth_client_connect_service_finish (client, res, NULL);
 
 	if (success == FALSE && g_timer_elapsed (data->timer, NULL) < CONNECT_TIMEOUT) {
-		if (bluetooth_client_connect_service(client, data->path, connect_callback, data) != FALSE)
-			return;
+		bluetooth_client_connect_service (client, data->path, TRUE, NULL, connect_callback, data);
 	}
 
 	if (success == FALSE)
-		g_message ("Failed to connect to device %s", data->path);
+		g_debug ("Failed to connect to device %s", data->path);
 
 	g_timer_destroy (data->timer);
 	g_free (data->path);
@@ -383,6 +384,8 @@ create_callback (BluetoothClient *_client,
 		 */
 		text = g_strdup_printf(_("Setting up '%s' failed"), target_name);
 
+		g_warning ("Setting up '%s' failed: %s", target_name, error->message);
+
 		gtk_label_set_markup(GTK_LABEL(label_failure), text);
 		return;
 	}
@@ -393,14 +396,8 @@ create_callback (BluetoothClient *_client,
 	data->path = g_strdup (path);
 	data->timer = g_timer_new ();
 
-	if (bluetooth_client_connect_service(client, path, connect_callback, data) != FALSE) {
-		gtk_assistant_set_current_page (window_assistant, PAGE_FINISHING);
-	} else {
-		gtk_assistant_set_current_page (window_assistant, PAGE_SUMMARY);
-		g_timer_destroy (data->timer);
-		g_free (data->path);
-		g_free (data);
-	}
+	bluetooth_client_connect_service (client, path, TRUE, NULL, connect_callback, data);
+	gtk_assistant_set_current_page (window_assistant, PAGE_FINISHING);
 }
 
 void
@@ -409,6 +406,7 @@ close_callback (GtkWidget *assistant,
 {
 	gtk_widget_destroy(assistant);
 
+	//FIXME doesn't work with a GtkApplication
 	gtk_main_quit();
 }
 
@@ -604,11 +602,16 @@ set_page_search_complete (void)
 gboolean
 entry_custom_event (GtkWidget *entry, GdkEventKey *event)
 {
+	gunichar c;
+
 	if (event->length == 0)
 		return FALSE;
 
-	if ((event->keyval >= GDK_KEY_0 && event->keyval <= GDK_KEY_9) ||
-	    (event->keyval >= GDK_KEY_KP_0 && event->keyval <= GDK_KEY_KP_9))
+	/* Not a printable character? */
+	c = gdk_keyval_to_unicode (event->keyval);
+	if (c == 0 ||
+	    g_unichar_iscntrl (c) ||
+	    g_unichar_isdigit (c))
 		return FALSE;
 
 	return TRUE;
@@ -617,6 +620,7 @@ entry_custom_event (GtkWidget *entry, GdkEventKey *event)
 void
 entry_custom_changed (GtkWidget *entry)
 {
+	g_free (user_pincode);
 	user_pincode = g_strdup (gtk_entry_get_text(GTK_ENTRY(entry)));
 	gtk_dialog_set_response_sensitive (GTK_DIALOG (pin_dialog),
 					   GTK_RESPONSE_ACCEPT,
@@ -742,39 +746,30 @@ pin_option_button_clicked (GtkButton *button,
 			       gpointer data)
 {
 	GtkWidget *radio;
-	char *oldpin;
-
-	oldpin = user_pincode;
-	user_pincode = NULL;
 
 	gtk_window_set_transient_for (GTK_WINDOW (pin_dialog),
 				      GTK_WINDOW (window_assistant));
 	gtk_window_present (GTK_WINDOW (pin_dialog));
 
 	/* When reopening, try to guess where the pincode was set */
-	if (oldpin == NULL)
+	if (user_pincode == NULL)
 		radio = radio_auto;
-	else if (g_str_equal (oldpin, "0000"))
+	else if (g_str_equal (user_pincode, "0000"))
 		radio = radio_0000;
-	else if (g_str_equal (oldpin, "1111"))
+	else if (g_str_equal (user_pincode, "1111"))
 		radio = radio_1111;
-	else if (g_str_equal (oldpin, "1234"))
+	else if (g_str_equal (user_pincode, "1234"))
 		radio = radio_1234;
-	else
+	else if (g_str_equal (user_pincode, "NULL"))
+		radio = radio_none;
+	else {
 		radio = radio_custom;
-
-	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (radio), TRUE);
-	if (radio == radio_custom)
-		gtk_entry_set_text (GTK_ENTRY (entry_custom), oldpin);
-
-	if (gtk_dialog_run (GTK_DIALOG (pin_dialog)) != GTK_RESPONSE_ACCEPT) {
-		g_free (user_pincode);
-		user_pincode = oldpin;
-	} else {
-		g_free (oldpin);
+		gtk_entry_set_text (GTK_ENTRY (entry_custom), user_pincode);
 	}
+	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (radio), TRUE);
 
-	gtk_widget_hide(pin_dialog);
+	gtk_dialog_run (GTK_DIALOG (pin_dialog));
+	gtk_widget_hide (pin_dialog);
 }
 
 static int
@@ -795,25 +790,8 @@ page_func (gint current_page,
 static GtkAssistant *
 create_wizard (void)
 {
-	const char *pages[] = {
-		"page_intro",
-		"page_search",
-		"page_connecting",
-		"page_setup",
-		"page_ssp_setup",
-		"page_failure",
-		"page_finishing",
-		"page_summary",
-	};
-	guint i;
-
 	GtkAssistant *assistant;
 	GError *err = NULL;
-	GtkWidget *combo, *page_intro;
-	GtkTreeModel *model;
-	GtkCellRenderer *renderer;
-	GdkPixbuf *pixbuf;
-	int height;
 
 	builder = gtk_builder_new ();
 	if (gtk_builder_add_from_file (builder, "wizard.ui", NULL) == 0) {
@@ -836,24 +814,7 @@ create_wizard (void)
 	gtk_widget_hide (button_close);
 	gtk_widget_hide (button_cancel);
 
-	/* Intro page */
-	combo = gtk_combo_box_new();
-
-	model = bluetooth_client_get_adapter_model(client);
-	gtk_combo_box_set_model(GTK_COMBO_BOX(combo), model);
-	g_object_unref(model);
-
-	gtk_combo_box_set_active(GTK_COMBO_BOX(combo), 0);
-	gtk_cell_layout_clear(GTK_CELL_LAYOUT(combo));
-
-	renderer = gtk_cell_renderer_text_new();
-	gtk_cell_layout_pack_start(GTK_CELL_LAYOUT(combo), renderer, TRUE);
-	gtk_cell_layout_set_attributes(GTK_CELL_LAYOUT(combo), renderer,
-					"text", BLUETOOTH_COLUMN_NAME, NULL);
-
-	page_intro = W("page_intro");
-	if (gtk_tree_model_iter_n_children(model, NULL) > 1)
-		gtk_box_pack_start(GTK_BOX(page_intro), combo, FALSE, FALSE, 0);
+	/* Intro page, nothing to do */
 
 	/* Search page */
 	page_search = W("page_search");
@@ -892,15 +853,6 @@ create_wizard (void)
 	label_summary = W("label_summary");
 	extra_config_vbox = W("extra_config_vbox");
 
-	/* Set page icons */
-	gtk_icon_size_lookup (GTK_ICON_SIZE_DIALOG, NULL, &height);
-	pixbuf = gtk_icon_theme_load_icon (gtk_icon_theme_get_default (),
-					   "bluetooth", height, 0, NULL);
-	for (i = 0; i < G_N_ELEMENTS (pages); i++)
-		gtk_assistant_set_page_header_image (assistant, W(pages[i]), pixbuf);
-	if (pixbuf != NULL)
-		g_object_unref (pixbuf);
-
 	/* PIN dialog */
 	pin_dialog = W("pin_dialog");
 	radio_auto = W("radio_auto");
